#include <ftw.h>    // http://man7.org/linux/man-pages/man3/ftw.3.html
#include <utime.h>  // http://linux.die.net/man/2/utime
#include <chrono>
#include <climits>
#include <cstdlib>  // http://man7.org/linux/man-pages/man3/realpath.3.html
#include <cstring>  // strlen
#include <iostream>
#include <string>
#include <thread>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;

unordered_map<string, int> filemap;
unordered_set<string> ignored_paths;

static int trigger_inotify_if_changed(const char *fpath, const struct stat *sb, int tflag, struct FTW *ftwbuf) {
  int ret = FTW_CONTINUE;
  
  char *abspath = realpath(fpath, NULL); // must be freed
  string filepath(fpath, strlen(fpath));
  if (abspath != NULL) {
    filepath.assign((const char *)abspath, strlen(abspath)); free(abspath);
  }

  if (ignored_paths.count(filepath)) {
    if (tflag == FTW_D) {
      ret = FTW_SKIP_SUBTREE;
    }
  } else if (tflag == FTW_F) {
    timespec mtime = sb->st_mtim;

    if (filemap.count(filepath)) {
      if (filemap[filepath] != mtime.tv_sec) {
        cout << filepath << " has changed at " << mtime.tv_sec << endl;

        // Touch the file to trigger an inotify event
        int success = utime(fpath, 0);
        if (success != 0) {
          cout << "Error touching " << filepath << endl;
        }

        // Get the time that the file now says it was modified
        struct stat attrib;
        stat(fpath, &attrib);

        // Update map to the new time
        filemap[filepath] = attrib.st_mtime;
      }
    } else {
      filemap.emplace(filepath, mtime.tv_sec);
      cout << "  " << filepath << ' ' << mtime.tv_sec << endl;
    }
  }

  return ret;
}

int main(int argc, const char *argv[]) {
  int flags = FTW_ACTIONRETVAL;
  int success = 0;

  const char *fpath = ".";
  char *abspath;

  vector<const char*> args(argv + 1, argv + argc);
  string filepath;

  args.emplace(args.begin(), "./node_modules");

  cout << "> Ignoring:" << endl;

  for (auto& arg: args) {
    abspath = realpath(arg, NULL);
    if (abspath != NULL) {
      filepath.assign((const char*)abspath, strlen(abspath)); free(abspath);
      ignored_paths.emplace(filepath);
      cout << "  " << filepath << endl;
    }
  }

  cout << "> Watching:" << endl;

  while(success != -1) {
    success = nftw(fpath, trigger_inotify_if_changed, 20, flags);
    std::this_thread::sleep_for(std::chrono::milliseconds(1000));
  }

  return success == -1 ? 1 : 0;
}
